/*******************************************************************
 *
 *	list.c V1.1
 *
 *	This file provides basic functions for linear double linked lists
 *	Most of this can be found in any computer book
 *	(c) 1998 Heinrich Wiedemann
 *	
 *
 *	History:	V1.0	Initial build
 *				V1.1	Added sorting
 *				V1.01	Fixed a bug in declarations (detected by HP-UX
 *						compiler, VC++ seems to accept every nonsense...)
 *
 ******************************************************************/


#include "list.h"
#include <malloc.h>

#define NULL ((void *)0)



/*******************************************************************
 *
 *	Constructs a list
 *
 *	Output:	list		Pointer to internal list-record
 *			List_Err_Ok if successful
 *
 ******************************************************************/

int ListConstruct(List *list)
{
	if((*list=malloc(sizeof(ListRec))) == NULL)
		return(List_Err_Out_Of_Memory);
	(*list)->head=(ListNode)NULL;
	(*list)->tail=(ListNode)NULL;
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	Destructs a list and all of its contents
 *
 *	Input:	list		the list
 *			callback	a user provided callback for deleting user-data
 *			user		a pointer to user-data
 *
 *	Output:	none
 *
 ******************************************************************/

void ListDestruct(List list, ListDestrCallback callback, void *user)
{
	ListNode cur, next;
	void *data;

	cur=list->head;
	while(cur)
	{
		next=cur->next;
		data=cur->data;
		if(callback)
			callback(data, user);
		free(cur);
		cur=next;
	}
	list->head=(ListNode)NULL;
	list->tail=(ListNode)NULL;
}



/*******************************************************************
 *
 *	Finds the node in the list, to which the same data-pointer exists
 *
 *	Input:	list		the list
 *			data		pointer to the same block of data
 *
 *	Output:	the list node. NULL, if it was not found
 *
 ******************************************************************/

ListNode ListNodeFind(List list, void *data)
{
	ListNode cur;

	cur=list->head;
	while(cur)
	{
		if(cur->data == data)
			return cur;
		cur=cur->next;
	}
	return((ListNode)NULL);
}



/*******************************************************************
 *
 *	Finds the node in the list with the same user-data
 *
 *	Input:	list		the list
 *			userdata	pointer to the user-data
 *			callback	a function, comparing node-data and user-data
 *
 *	Output:	the list node. NULL, if it was not found
 *
 ******************************************************************/

ListNode ListNodeSearch(List list, void *userdata, ListSearchCallback callback)
{
	ListNode cur;

	cur=list->head;
	while(cur)
	{
		if(callback)
			if(callback(cur->data, userdata) == 0)
				return(cur);
		cur=cur->next;
	}
	return((ListNode)NULL);
}



/*******************************************************************
 *
 *	Adds a node to the end of the list
 *
 *	Input:	list		the list
 *			data		pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

int ListNodeAdd(List list, void *data)
{
	ListNode before;
	ListNode node;


	before=list->tail;
	if((node=malloc(sizeof(ListNodeRec))) == NULL)
		return(List_Err_Out_Of_Memory);
	node->next=(ListNode)NULL;
	node->prev=before;
	node->data=data;
	if(before)
		before->next=node;
	else
		list->head=node;
	list->tail=node;
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	Inserts a node at the head of the list
 *
 *	Input:	list		the list
 *			data		pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

int ListNodeInsert(List list, void *data)
{
	ListNode after;
	ListNode node;
		
	after=list->head;
	if((node=malloc(sizeof(ListNodeRec))) == NULL)
		return(List_Err_Out_Of_Memory);
	node->next=after;
	node->prev=(ListNode)NULL;
	node->data=data;
	if(!after)
		list->tail=node;
	else
		after->prev=node;
	list->head=node;
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	Removes a node from the list
 *	User data has to be deleted by the user
 *
 *	Input:	list		the list
 *			node		the node to delete
 *
 *	Output:	none
 *
 ******************************************************************/

void ListNodeRemove(List list, ListNode node)
{
	ListNode before, after;

	before=node->prev;
	after=node->next;
	if(before)
		before->next=after;
	else
		list->head=after;
	if(after)
		after->prev=before;
	else
		list->tail=before;
	free(node);
}



/*******************************************************************
 *
 *	Moves a node from the list to the head (good for LRU-handling)
 *	User data has to be deleted by the user
 *
 *	Input:	list		the list
 *			node		the node to delete
 *
 *	Output:	none
 *
 ******************************************************************/

void ListNodeToHead(List list, ListNode node)
{
	ListNode before, after;

	before=node->prev;
	after=node->next;

	/* check wether we're already on top of the list */
	if(!before)
		return;
	before->next=after;
	if(after)
		after->prev=before;
	else
		list->tail=before;
	node->prev=(ListNode)NULL;
	node->next=list->head;
	list->head=node;
}



/*******************************************************************
 *
 *	Walks through the list and visits a callback function for each node
 *
 *	Input:	list		the list
 *			callback	the callback function, has to return List_Err_Ok
 *						otherwise the walking will be terminated
 *			up_down		from head..tail or tail..head (List_Walk_Up / _Down)
 *			user		a pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

int ListWalk(List list, ListWalkCallback callback, int up_down, void *user)
{
	ListNode cur, next, prev;
	int err;

	if(up_down == List_Walk_Up)
	{
		cur=list->head;
		while(cur)
		{
			next=cur->next;
			if((err=callback(cur->data, user)) != List_Err_Ok)
				return(err);
			cur=next;
		}
	}
	else
	{
		cur=list->tail;
		while(cur)
		{
			prev=cur->prev;
			if((err=callback(cur->data, user)) != List_Err_Ok)
				return(err);
			cur=prev;
		}
	}
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	Sort a list using Bubblesort
 *
 *	Input:	list		the list
 *			callback	the user-defined comparing-function (<0 - 0 - >0)
 *			up_down		up or down sorting (List_Sort_Up / List_Sort_Down)
 *
 *	Output:	none
 *
 ******************************************************************/

void ListSort(List list, ListSortCallback compare, int up_down)
{
	ListNode cur;
	ListNode now, max;
	int cmp;
	void *swap;

	cur=list->head;
	while(cur)
	{
		max=cur;
		now=cur->next;
		while(now)
		{
			cmp=compare(max->data, now->data);
			if((cmp>0 && up_down==List_Sort_Up) || (cmp<0 && up_down==List_Sort_Down))
				max=now;
			now=now->next;
		}
		swap=cur->data;
		cur->data=max->data;
		max->data=swap;
		cur=cur->next;
	}
}
