/*******************************************************************
 *
 *	list.h V1.1
 *
 *	This file provides basic functions for linear double linked lists
 *	Most of this can be found in any computer book
 *	(c) 1998 Heinrich Wiedemann
 *	
 *
 *	History:	V1.0	Initial build
 *				V1.1	Added sorting
 *				V1.01	Fixed a bug in declarations (detected by HP-UX
 *						compiler, VC++ seems to accept every nonsense...)
 *
 ******************************************************************/

#define List_Err_Ok				0
#define List_Err_Out_Of_Memory	1


#define List_Sort_Up	0
#define List_Sort_Down	1

#define List_Walk_Up	0
#define List_Walk_Down	1



typedef int ListError;

/*******************************************************************
 *
 *	ListNode
 *	A ListNode is a handle to a single list element
 *
 ******************************************************************/

typedef struct ListNodeRec_ *ListNode;



/*******************************************************************
 *
 *	List
 *	A handle to a list record (see ListRec)
 *
 ******************************************************************/

typedef struct ListRec_ *List;



/*******************************************************************
 *
 *	ListNodeRec
 *	A structure used to hold a single list element
 *
 ******************************************************************/

typedef struct ListNodeRec_
{
	ListNode prev;		/* previous element in the list. NULL if first	*/
	ListNode next;		/* next element in the list. NULL if last		*/
	void	 *data;		/* typeless pointer to the listed object		*/
}
ListNodeRec;



/*******************************************************************
 *
 *	ListRec
 *	A structure used to hold a simple doubly-linked list
 *
 ******************************************************************/

typedef struct ListRec_
{
	ListNode head;		/* head (first element) of doubly-linked list	*/
	ListNode tail;		/* tail (last  element) of doubly-linked list	*/
}
ListRec;


#define LIST_IS_EMPTY(list)  ( (list).head == 0 )



/*******************************************************************
 *
 *	Constructs a list
 *
 *	Output:	list		Pointer to internal list-record
 *			List_Err_Ok if successful
 *
 ******************************************************************/

int ListConstruct(List *list);



/*******************************************************************
 *
 *	Destructs a list and all of its contents
 *
 *	Input:	list		the list
 *			callback	a user provided callback for deleting user-data
 *			user		a pointer to user-data
 *
 *	Output:	none
 *
 ******************************************************************/

typedef void (*ListDestrCallback)(void *data, void *user);

void ListDestruct(List list, ListDestrCallback callback, void *user);



/*******************************************************************
 *
 *	Finds the node in the list, to which the same data-pointer exists
 *
 *	Input:	list		the list
 *			data		pointer to the same block of data
 *
 *	Output:	the list node. NULL, if it was not found
 *
 ******************************************************************/

ListNode ListNodeFind(List list, void *data);



/*******************************************************************
 *
 *	Finds the node in the list with the same user-data
 *
 *	Input:	list		the list
 *			userdata	pointer to the user-data
 *			callback	a function, comparing node-data and user-data
 *
 *	Output:	the list node. NULL, if it was not found
 *
 ******************************************************************/

typedef int (*ListSearchCallback)(void *nodedata, void *userdata);

ListNode ListNodeSearch(List list, void *data, ListSearchCallback callback);



/*******************************************************************
 *
 *	Adds a node to the end of the list
 *
 *	Input:	list		the list
 *			data		pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

int ListNodeAdd(List list, void *data);



/*******************************************************************
 *
 *	Inserts a node at the head of the list
 *
 *	Input:	list		the list
 *			data		pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

int ListNodeInsert(List list, void *data);



/*******************************************************************
 *
 *	Removes a node from the list
 *	User data has to be deleted by the user
 *
 *	Input:	list		the list
 *			node		the node to delete
 *
 *	Output:	none
 *
 ******************************************************************/

void ListNodeRemove(List list, ListNode node);



/*******************************************************************
 *
 *	Moves a node from the list to the head (good for LRU-handling)
 *	User data has to be deleted by the user
 *
 *	Input:	list		the list
 *			node		the node to delete
 *
 *	Output:	none
 *
 ******************************************************************/

void ListNodeToHead(List list, ListNode node);


/*******************************************************************
 *
 *	Walks through the list and visits a callback function for each node
 *
 *	Input:	list		the list
 *			callback	the callback function, has to return List_Err_Ok
 *						otherwise the walking will be terminated
 *			up_down		from head..tail or tail..head (List_Walk_Up / _Down)
 *			user		a pointer to user-data
 *
 *	Output:	List_Err_Ok if successful
 *
 ******************************************************************/

typedef int (*ListWalkCallback)(void *data, void *user);

int ListWalk(List list, ListWalkCallback callback, int up_down, void *user);



/*******************************************************************
 *
 *	Sort a list using Bubblesort
 *
 *	Input:	list		the list
 *			callback	the user-defined comparing-function (<0 - 0 - >0)
 *			up_down		up or down sorting (List_Sort_Up / List_Sort_Down)
 *
 *	Output:	none
 *
 ******************************************************************/

typedef int (*ListSortCallback)(void *max, void *current);

void ListSort(List list, ListSortCallback compare, int up_down);
