/*******************************************************************
 *
 *	ugzipc V1.0.5
 *
 *	Zip components of an Unigraphics assembly
 *	(c) 1999-2004 Heinrich Wiedemann
 *	http://www.w-eng.de/ugzipc.htm
 *
 *	Using zlib 1.1.4 (c) 1995-1998 Jean-loup Gailly and Mark Adler
 *	See http://www.gzip.org/zlib/ for more information
 *
 *	History:	V1.0	Initial design
 *				V1.0.1	Removed inconsistencies of dialog file
 *						between UG V15 and V16, no changes in the
 *						code (beside a workaround in UGZ_constr_cb)
 *				V1.0.2	Corrected a minor bug, which prevented use
 *						from UG/Menuscript
 *				V1.0.3	Included modifications by Steven L. Alexander/
 *						Delphi Automotive Systems for Unix-compatiblity
 *				V1.0.4	Issues a warning when UG being run under iMAN or
 *						UG/Manager-environment - not yet supported
 *						volunteers welcome
 *				V1.0.5	Removed compiler error/warnings on HP-UX -
 *						contributed by Rafael Portela/Sony Ericsson.
 *						Using zlib 1.1.4
 *
 ******************************************************************/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>

#include <uf.h>
#include <uf_defs.h>
#include <uf_exit.h>
#include <uf_ui.h>
#include <uf_styler.h>
#include <uf_mb.h>
#include <uf_cfi.h>
#include <uf_part.h>
#include <uf_assem.h>

#include <zip.h>

#include "list.h"
#include "ugzipc.h"


#define UGZ_CB_COUNT ( 8 + 1 ) /* Add 1 for the terminator */
 
/*--------------------------------------------------------------------------
The following structure defines the callback entries used by the       
styler file.  This structure MUST be passed into the user function,    
UF_STYLER_create_dialog along with UGZ_CB_COUNT.                       
--------------------------------------------------------------------------*/
static UF_STYLER_callback_info_t UGZ_cbs[UGZ_CB_COUNT] = 
{
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_CONSTRUCTOR_CB  , 0, UGZ_constr_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_OK_CB           , 0, UGZ_ok_cb},
 {UF_STYLER_DIALOG_INDEX, UF_STYLER_APPLY_CB        , 0, UGZ_apply_cb},
 {UGZ_SOURCE            , UF_STYLER_VALUE_CHANGED_CB, 0, UGZ_assembly_cb},
 {UGZ_PART_SEL          , UF_STYLER_ACTIVATE_CB     , 0, UGZ_part_sel_cb},
 {UGZ_PARTFILE          , UF_STYLER_ACTIVATE_CB     , 0, UGZ_part_cb},
 {UGZ_ZIP_SEL           , UF_STYLER_ACTIVATE_CB     , 0, UGZ_zip_sel_cb},
 {UGZ_ZIPFILE           , UF_STYLER_ACTIVATE_CB     , 0, UGZ_zip_cb},
 {UF_STYLER_NULL_OBJECT, UF_STYLER_NO_CB, 0, 0 }
};

static UF_MB_styler_actions_t actions[] = {
    { "ugzipc.dlg",  NULL,   UGZ_cbs,  UF_MB_STYLER_IS_NOT_TOP },
    { NULL,  NULL,  NULL,  0 } /* This is a NULL terminated list */
};



#define RW_BUFFERSIZE (16384)		/* for filecopy-operation */



/*******************************************************************
 *
 *	Display message due to UG error
 *
 *	Input:	err			UG error code
 *
 *	Output:	none
 *
 ******************************************************************/

void UGError(int err)
{
	char err_msg[UF_UI_MAX_STRING_LEN+1];
	char fail_msg[133];

	if(err)
	{
		UF_get_fail_message(err, fail_msg);
		sprintf(err_msg, "UG - error:*%s", fail_msg);
		err_msg[132]='\0';
		uc1601(err_msg, TRUE);
	}
}



/*******************************************************************
 *
 *	Display message due to UG error and return FALSE
 *
 *	Input:	err			UG error code
 *
 *	Output:	False
 *
 ******************************************************************/

int UGErrorFalse(int err)
{
	UGError(err);
	return(FALSE);
}



/*******************************************************************
 *
 *	Display message due to OS error
 *
 *	Input:	msg			error message
 *
 *	Output:	none
 *
 ******************************************************************/

void OSError(char *msg)
{
	char err_msg[UF_UI_MAX_STRING_LEN+1];
	int er;

	if((er=UF_DISP_regenerate_display()) != 0)
		UGError(er); 
	sprintf(err_msg, "OS - error:*%s.", msg);
	err_msg[132]='\0';
	uc1601(err_msg, TRUE);
}



/*******************************************************************
 *
 *	Gets date/time (modification) of a file
 *	This is done strictly with UG-cfi for portability (NT vs. UNIX)
 *
 *	Input:	fname		Filename
 *
 *	Output:	tm_zip		Date/time
 *			dt			unmodified
 *
 ******************************************************************/

void filetime(char *fname, tm_zip *tmzip, uLong *dt)
{
	int err;
	int cdate[2], mdate[2], ldate[2];
	char datestg[20], timestg[20];

	if((err=uc4564(fname, 2, 16)) != 0)			/* read file header info: 2: .prt-file; 16: only date/time	*/
		UGError(err);
	if((err=uc4606(cdate, mdate, ldate)) != 0)	/* obtain creation, modification and last access date/time */
		UGError(err);
	if((err=uc4582(mdate, 5, datestg, timestg)) != 0)	/* convert numbers to mm/dd/yyyy and hh:mm (modified) */
		UGError(err);
	sscanf(datestg, "%d/%d/%d", &tmzip->tm_mon, &tmzip->tm_mday, &tmzip->tm_year);
	tmzip->tm_mon--;							/* adjust month */
	sscanf(timestg, "%d:%d", &tmzip->tm_hour, &tmzip->tm_min);
	tmzip->tm_sec=0;
}



/*******************************************************************
 *
 *	Converts a NT filename to zip-conventions by removing
 *	drivename and converting '\' to '/'
 *	this function will not disturb UNIX-filenames (hopefully...)
 *
 *	Input:	nt_fname	NT-filename
 *
 *	Output:	ux_name		UNIX-filename
 *
 ******************************************************************/

void PolishFilename(char *nt_name, char *ux_name)
{
	char *s;
	
	s=nt_name;
	if(isalpha(*s) && *(s+1)==':')	/* skip 'driveletter:' */
		s+=2;
	if(*s=='\\' || *s=='/')			/* skip leading '\' or '/' */
		s++;
	strcpy(ux_name, s);
	for(s=ux_name ; *s; s++)		/* change all '\' to '/' */
		if(*s == '\\')
			*s='/';
}



/*******************************************************************
 *
 *	Compress one part-component file into a zip-file
 *	adopted from minizip.c
 *
 *	Input:	files		array of component filenames
 *			count		number of filenames
 *			zfname		filename of zip-file
 *			compr		level of compression (see zlib.h)
 *			save_dir	save full filespec, if TRUE
 *
 *	Output:	error code
 *
 ******************************************************************/

int ZipOneComponent(zipFile zf, char *fzip, int compr, int save_dir, void *buf)
{
	FILE *fin;
	int size_read;
	char filenameinzip[MAX_FSPEC_SIZE+1];
	char msg[MAX_FSPEC_SIZE+80];
	char dspec[MAX_FSPEC_SIZE+1];
	zip_fileinfo zi;
	int err, ugerr;

	err=ZIP_OK;
	if(save_dir)
		PolishFilename(fzip, filenameinzip);
	else
		if((ugerr=uc4576(fzip, 2, dspec, filenameinzip)) != 0)	/* split fname into directory and file-spec */
			UGError(ugerr);
	zi.dosDate=0;
	zi.internal_fa=0;
	zi.external_fa=0;
	filetime(fzip, &zi.tmz_date, &zi.dosDate);

	sprintf(msg, "  Compressing %s\n", fzip);
	ugerr=UF_UI_write_listing_window(msg);
	err=zipOpenNewFileInZip(zf, filenameinzip, &zi, NULL, 0, NULL, 0, NULL, 
							(compr != 0) ? Z_DEFLATED : 0, compr);
	if(err != ZIP_OK)
	{
		sprintf(msg, "    Error in creating %s in zipfile\n", filenameinzip);
		ugerr=UF_UI_write_listing_window(msg);
	}
	else
	{
		fin=fopen(fzip, "rb");
		if(fin == NULL)
		{
			err=ZIP_ERRNO;
			sprintf(msg, "    Error in opening %s for reading\n", fzip);
			ugerr=UF_UI_write_listing_window(msg);
		}
	}
	if(err == ZIP_OK)
		do
		{
			err=ZIP_OK;
			size_read=fread(buf, 1, RW_BUFFERSIZE, fin);
			if(size_read < RW_BUFFERSIZE)
				if(feof(fin) == 0)
				{
					sprintf(msg, "    Error in reading %s\n", fzip);
					ugerr=UF_UI_write_listing_window(msg);
					err=ZIP_ERRNO;
				}
			if(size_read>0)
			{
				err=zipWriteInFileInZip(zf, buf, size_read);
				if(err < 0)
				{
					sprintf(msg, "    Error in writing %s in zipfile\n", filenameinzip);
					ugerr=UF_UI_write_listing_window(msg);
				}
			}
		}
		while((err == ZIP_OK) && (size_read>0));
	if(fin != NULL)
		fclose(fin);
	if(err < 0)
		err=ZIP_ERRNO;
	else
	{                    
		err=zipCloseFileInZip(zf);
		if (err!=ZIP_OK)
		{
			sprintf(msg, "    Error in closing %s in zipfile\n", filenameinzip);
			ugerr=UF_UI_write_listing_window(msg);
		}
	}
	return(err);
}



/*******************************************************************
 *
 *	Compress a bunch of component files into a zip-file
 *
 *	Input:	files		array of component filenames
 *			count		number of filenames
 *			zfname		filename of zip-file
 *			compr		level of compression (see zlib.h)
 *			save_dir	save full filespec, if TRUE
 *
 *	Output:	error code
 *
 ******************************************************************/

int ZipComponents(char **files, int count, char *zfname, int compr, int save_dir)
{
	int i;
	int err=ZIP_OK;
	void *buf=NULL;
	zipFile zf;
	int errclose;
	int ugerr;
	logical is_open;
	char msg[MAX_FSPEC_SIZE+80];
	char *comment="";

	ugerr=UF_UI_is_listing_window_open(&is_open);
	if(! is_open)
		ugerr=UF_UI_open_listing_window();
	ugerr=UF_UI_write_listing_window("UGZipC V1.05, (c) wiedemann engineering 1999-2004, www.w-eng.de\n");
	ugerr=UF_UI_write_listing_window("-------------------- UGZipC Compression Log -------------------\n");

	buf=malloc(RW_BUFFERSIZE);
	if(buf == NULL)
	{
		ugerr=UF_UI_write_listing_window("Error allocating memory\n");
		return(ZIP_INTERNALERROR);
	}
	zf=zipOpen(zfname, 0);		/* 0: don't append */
	if(zf == NULL)
	{
		sprintf(msg, "Error in creating %s\n", zfname);
		ugerr=UF_UI_write_listing_window(msg);
		err=ZIP_ERRNO;
	}
	else 
	{
		sprintf(msg, "Zip-file %s created\n", zfname);
		ugerr=UF_UI_write_listing_window(msg);
	}
	for (i=0; (i<count) && (err==ZIP_OK); i++)
		err=ZipOneComponent(zf, files[i], compr, save_dir, buf);
	errclose=zipClose(zf, comment);
	if(errclose != ZIP_OK)
	{
		sprintf(msg, "Error in closing %s\n", zfname);
		ugerr=UF_UI_write_listing_window(msg);
	}
	else
		ugerr=UF_UI_write_listing_window("Zip-file closed\n");
	free(buf);
	return(err);  
}



/*******************************************************************
 *
 *	Performs the zip with dialog parameters, when Ok or Apply is pressed
 *
 *	Input:	dialog_id	id of dialog-box
 *
 *	Output:	TRUE: successful, FALSE: error
 *
 ******************************************************************/

int DoTheZIP(int dialog_id)
{
	UF_STYLER_item_value_type_t dataz, datac;
	int compr, overwrite, save_dir;
	int err;

	dataz.item_id=UGZ_CLEVEL;
	dataz.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &dataz)) != 0)	/* get compression level */
		UGError(err);
	switch(dataz.value.integer)
	{
		case 0:	compr=Z_NO_COMPRESSION;
				break;
		case 1:	compr=Z_BEST_SPEED;
				break;
		case 2:	compr=Z_DEFAULT_COMPRESSION;
				break;
		case 3:	compr=Z_BEST_COMPRESSION;
				break;
		default:compr=Z_DEFAULT_COMPRESSION;
				break;
	}
	dataz.item_id=UGZ_OVERWRITE;
	dataz.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &dataz)) != 0)	/* get overwrite status */
		UGError(err);
	overwrite=dataz.value.integer == 1;

	dataz.item_id=UGZ_DIR_INFO;
	dataz.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &dataz)) != 0)	/* get Save DirInfo status */
		UGError(err);
	save_dir=dataz.value.integer == 1;

	dataz.item_id=UGZ_ZIPFILE;
	dataz.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &dataz)) != 0)	/* get zip file name */
		UGError(err);

	err=uc4560(dataz.value.string, 0);	/* check, if zip-file exists */
	if(err < 0)
		return(UGErrorFalse(err));
	else
		if(err == 0 && overwrite == FALSE)		/* zip-file exists && not overwrite */
		{
			uc1601("Zip-file already exists", 1);
			return(FALSE); 
		}
		else
		{
			datac.item_id=UGZ_CLIST;
			datac.item_attr=UF_STYLER_VALUE;
			datac.indicator=UF_STYLER_STRING_PTR_VALUE;
			if((err=UF_STYLER_ask_value(dialog_id, &datac)) != 0)	/* get array of components */
				UGError(err);
			err=ZipComponents(datac.value.strings, datac.count, dataz.value.string, compr, save_dir);
			UF_STYLER_free_value(&datac);
		}
    return(TRUE);                           
}



/*******************************************************************
 *
 *	Add filename of a component to the list
 *
 *	Input:	fname	filename of component
 *
 *	Output:	none
 *
 ******************************************************************/

void AddComponentToList(List clist, char *fname)
{
	ListNode node;
	char *s;

	node=ListNodeSearch(clist, (void *)fname, (ListSearchCallback)&strcmp);
	if(node == NULL)	/* if not already in list, add it */
	{
		s=malloc(strlen(fname)+1);
		if(s == NULL)
			OSError("Error allocating memory");
		else
		{
			strcpy(s, fname);
			ListNodeAdd(clist, s);

		}
	}
}



/*******************************************************************
 *
 *	Scans an assembly recursively and adds its components to a list
 *	Similar to code of Daniel Dignam/UGS in ugsolutions.languages...
 *
 *	Input:	clist		the component list
 *			part_occ	the current occurence
 *
 *	Output:	none
 *
 ******************************************************************/

void ScanSubAssembly(List clist, tag_t comp)
{
	char fname[MAX_FSPEC_SIZE+1];
	char rset_name[MAX_ENTITY_NAME_SIZE+1];
	char comp_name[MAX_ENTITY_NAME_SIZE+1];
	double org[3];
	double csys[9];
	double transform[4][4];
	tag_t *children;
	int i, n_children;
	int err;

	if((err=UF_ASSEM_ask_component_data(comp, fname, rset_name, comp_name, org, csys, transform)) != 0)
		UGError(err);
	else
	{
		AddComponentToList(clist, fname);
		n_children=UF_ASSEM_ask_part_occ_children(comp, &children);
		for(i=0; i < n_children; i++)
			ScanSubAssembly(clist, children[i]);
		UF_free(children);
	}
}



/*******************************************************************
 *
 *	List-callback: Delete attached data of a component list entry
 *
 *	Input:	data	here: filename of component
 *			user	unused
 *
 *	Output:	none
 *
 ******************************************************************/

void DeleteComp(void *data, void *user)
{
	free(data);
}



/*******************************************************************
 *
 *	List-callback: Check, if part-file exists
 *
 *	Input:	ldata	here: filename of component
 *			user	here: pointer to info: log-line already displayed
 *
 *	Output:	none
 *
 ******************************************************************/

int CheckCList(void *ldata, void *user)
{
	char *fname;
	int *open_log;
	logical is_open;
	char msg[MAX_FSPEC_SIZE+80];
	int err;

	fname=(char *)ldata;
	open_log=(int *)user;
	err=uc4560(fname, 2);		/* check, if part-file exists */
	if(err < 0)
		UGError(err);
	else
		if(err != 0)
		{
			err=UF_UI_is_listing_window_open(&is_open);
			if(! is_open)
				err=UF_UI_open_listing_window();
			if(! *open_log)
			{
				err=UF_UI_write_listing_window("------------------- UGZipC Assembly Open Log ------------------\n");
				*open_log=TRUE;
			}
			sprintf(msg, "File %s not found\n", fname);
			err=UF_UI_write_listing_window(msg);
			strcpy(fname, "");
		}
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	List-callback: Insert an entry into the dialog component-list
 *
 *	Input:	ldata	here: filename of component
 *			user	here: pointer to dialog-id
 *
 *	Output:	none
 *
 ******************************************************************/

int InsertCList(void *ldata, void *user)
{
	int dialog_id;
	UF_STYLER_item_value_type_t data;
	int err;

	dialog_id=*((int *)user);
	if(*(char *)ldata)
	{
		data.item_id=UGZ_CLIST;
		data.item_attr=UF_STYLER_LIST_INSERT;
		data.subitem_index=0;
		data.indicator=UF_STYLER_STRING_VALUE;
		data.value.string=(char *)ldata;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* add entry to start of component list */
			UGError(err);
		data.item_attr= UF_STYLER_VALUE;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* and mark it as selected */
			UGError(err);
	}
	return(List_Err_Ok);
}



/*******************************************************************
 *
 *	Fill the dialogs component list
 *
 *	Input:	dialog_id	id of dialog-box
 *			is_open		TRUE if work-part, FALSE if selected file
 *			root_part	tag of work-part, if is_open==TRUE
 *			root_fname	filename, if is_open==FALSE
 *
 *	Output:	TRUE if successful, FALSE if error
 *
 ******************************************************************/

int FillComponentList(int dialog_id, int is_open, tag_t root_part, char *root_fname)
{
	tag_t root_occ;
	UF_PART_load_status_t error_status;
	char work_fname[MAX_FSPEC_SIZE+1];
	tag_t work_part;
	int i;
	UF_ASSEM_options_t options;
	UF_STYLER_item_value_type_t data;
	logical l_is_open;
	char msg[MAX_FSPEC_SIZE+80];
	char fail_msg[MAX_FSPEC_SIZE+80];
	List comp_list;
	int old_failure, old_loadcomp;
	int open_log=FALSE;
	int err, err_open;
	
	if(! is_open)		/* filename was selected */
		if((work_part=UF_ASSEM_ask_work_part()) != NULL_TAG)	/* Check, if selected file is current work part */
		{
			UF_PART_ask_part_name(work_part, work_fname);
			is_open=(strcmp(root_fname, work_fname) == 0);
			if(is_open)
				root_part=work_part;
		}
	if(! is_open)
	{
		if((err=UF_ASSEM_ask_assem_options(&options)) != 0)
			UGError(err);
		old_failure=options.failure_action;						/* save current load options */
		old_loadcomp=options.load_components;
		options.failure_action=UF_ASSEM_dont_abort_on_failure;
		options.load_components=UF_ASSEM_dont_load_components;
		if((err=UF_ASSEM_set_assem_options(&options)) != 0)		/* set temporary load-options */
			UGError(err);
		err_open=UF_PART_open_quiet(root_fname, &root_part, &error_status);

		options.failure_action=old_failure;
		options.load_components=old_loadcomp;
		if((err=UF_ASSEM_set_assem_options(&options)) != 0)		/* restore old load-options */
			UGError(err);

		if(error_status.n_parts > 0)
		{
			err=UF_UI_is_listing_window_open(&l_is_open);
			if(! l_is_open)
				err=UF_UI_open_listing_window();
			err=UF_UI_write_listing_window("------------------- UGZipC Assembly Open Log ------------------\n");
			open_log=TRUE;
			sprintf(msg, "Warnings while opening %s :\n", root_fname);
			err=UF_UI_write_listing_window(msg);
			for(i=0; i < error_status.n_parts; i++)
			{
				UF_get_fail_message(error_status.statuses[i], fail_msg);
				sprintf(msg, "  %s - %s\n", error_status.file_names[i], fail_msg);
				err=UF_UI_write_listing_window(msg);
			}
			UF_free_string_array(error_status.n_parts, error_status.file_names);
			UF_free(error_status.statuses);
		}
		if(err_open != 0)
			return(UGErrorFalse(err_open));
	}
	if(ListConstruct(&comp_list) != List_Err_Ok)
		printf("error ListConstruct\n");
	if((root_occ=UF_ASSEM_ask_root_part_occ(root_part)) != NULL_TAG)
		ScanSubAssembly(comp_list, root_occ);
	else
		AddComponentToList(comp_list, root_fname);		/* no sub-components - add only selected file */

	ListWalk(comp_list, &CheckCList, List_Walk_Up, (void *)&open_log);		/* Check, if part files exist */

	ListSort(comp_list, (ListSortCallback)&strcmp, List_Sort_Up);								/* Sort list */
	data.item_id=UGZ_CLIST;
	data.item_attr=UF_STYLER_LIST_DELETE;
	data.subitem_index=UF_STYLER_NO_SUB_INDEX;
	if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)					/* clear dialog component list */
		UGError(err);
	ListWalk(comp_list, &InsertCList, List_Walk_Down, (void *)&dialog_id);	/* add to dialog component list */
	ListDestruct(comp_list, &DeleteComp, NULL);
	if(! is_open)
		if((err=UF_PART_close(root_part, 0, 1)) != 0)
			UGError(err);
	return(TRUE);
}



/*******************************************************************
 *
 *	Set sensitivity of Ok and Apply button
 *	according to availability of partfile-name and zipfile-name
 *
 *	Input:	dialog_id	id of dialog-box
 *
 *	Output:	none
 *
 ******************************************************************/

void UpdateOkApply(int dialog_id)
{
	UF_STYLER_item_value_type_t data1, data2;
	int err;

	data1.item_id=UGZ_PARTFILE;
	data1.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &data1)) != 0)	/* get part file name */
		UGError(err);
	data2.item_id=UGZ_ZIPFILE;
	data2.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &data2)) != 0)	/* get zip file name */
		UGError(err);

	data1.value.integer=*data1.value.string && *data2.value.string;
	data1.item_id=UF_STYLER_NAV_INDEX;						/* part and zip-file available ? */
	data1.subitem_index=UF_STYLER_OK_INDEX;
	data1.item_attr=UF_STYLER_SENSITIVITY;
	if((err=UF_STYLER_set_value(dialog_id, &data1)) != 0)	/* set Ok button on/off */
		UGError(err);
	data1.subitem_index=UF_STYLER_APPLY_INDEX;
	if((err=UF_STYLER_set_value(dialog_id, &data1)) != 0)	/* set Apply button on/off */
		UGError(err);
}



/*******************************************************************
 *
 *	Callback-function for dialog-construction
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Continue dialog
 *
 ******************************************************************/

int UGZ_constr_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	UF_STYLER_item_value_type_t data;
	logical is_active;
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);

	if((err=UF_is_ugmanager_active(&is_active)) != 0)
		UGError(err);
	if(is_active)
		uc1601("UGZipC does not support UG when being run under iMAN or UG/Manager-environment.*Press Cancel to terminate.", 1);
	if(UF_ASSEM_ask_work_part() == NULL_TAG)	/* if no current work part */
	{
		data.item_id=UGZ_SOURCE;
		data.item_attr=UF_STYLER_VALUE;
		data.subitem_index=0;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* set source to 'Select Part' */
			UGError(err);
		data.item_attr=UF_STYLER_SENSITIVITY;
		data.subitem_index=1;
		data.value.integer=FALSE;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* disable 'Work Part' radio-button */
			UGError(err);
	}
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for Assembly-radio-button
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_assembly_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	UF_STYLER_item_value_type_t data;
	tag_t work_part;
	char fname[MAX_FSPEC_SIZE+1];
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);

	data.item_id=UGZ_PARTFILE;
	data.item_attr=UF_STYLER_SENSITIVITY;
	if(callback_data->value.integer == 0)					/* Select part or Work part ? */
		data.value.integer=TRUE;
	else
		data.value.integer=FALSE;
	if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* switch part file line on/off */
		UGError(err);
	data.item_id=UGZ_PART_SEL;
	if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* switch part select button on/off */
		UGError(err);
	if(callback_data->value.integer == 0)
	{
		data.item_id=UGZ_PARTFILE;								/* Select part clicked */
		data.item_attr=UF_STYLER_VALUE;
		data.value.string="";
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* clear part file name */
			UGError(err);
		data.item_id=UGZ_CLIST;
		data.item_attr=UF_STYLER_LIST_DELETE;
		data.subitem_index=UF_STYLER_NO_SUB_INDEX;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* clear component list */
			UGError(err);
	}
	else
		if((work_part=UF_ASSEM_ask_work_part()) != NULL_TAG)	/* Work part clicked */
		{
			UF_PART_ask_part_name(work_part, fname);
			data.item_id=UGZ_PARTFILE;
			data.item_attr=UF_STYLER_VALUE;
			data.value.string=fname;
			if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* place work part filename in input line */
				UGError(err);
			FillComponentList(dialog_id, TRUE, work_part, fname);
		}
	UpdateOkApply(dialog_id);
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for part-selection-line
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_part_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	UF_STYLER_item_value_type_t data;
	tag_t work_part=NULL_TAG;
	int is_open=FALSE;
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);
	err=uc4560(callback_data->value.string, 2);	/* check, if file exists; 2: part file */
	if(err < 0)
		UGError(err);
	else
		if(err == 0)		/* if file exists, fill component list */
			FillComponentList(dialog_id, FALSE, NULL_TAG, callback_data->value.string);
		else
		{
			data.item_id=UGZ_CLIST;									/* file does not exist */
			data.item_attr=UF_STYLER_LIST_DELETE;
			data.subitem_index=UF_STYLER_NO_SUB_INDEX;
			if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* clear component list */
				UGError(err);
			uc1601("Selected part file does not exist", 1);			/* display a message box */
		}
	UpdateOkApply(dialog_id);
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for part-select-button
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_part_sel_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	char fname[MAX_FSPEC_SIZE+1]="";	/* initial value for filebox */
	logical use_disp_file=FALSE;
	int resp;
	UF_STYLER_item_value_type_t data;
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);
	if((err=UF_UI_ask_open_part_filename(fname, &use_disp_file, &resp)) != 0)
		UGError(err);
	if(resp == UF_UI_OK)
	{
		data.item_id=UGZ_PARTFILE;
		data.item_attr=UF_STYLER_VALUE;
		data.value.string=fname;
		if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* place selected filename in input line */
			UGError(err);
		UpdateOkApply(dialog_id);
		FillComponentList(dialog_id, FALSE, NULL_TAG, fname);
	}
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for zip-selection-line
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_zip_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);
	if((err=uc4570(callback_data->value.string, 0)) != 0)	/* Check, if filename valid */
		UGError(err);
	UpdateOkApply(dialog_id);
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for zip-select-button
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_zip_sel_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	UF_STYLER_item_value_type_t data;
	char fname[MAX_FSPEC_SIZE+1]="";
	char dspec[MAX_FSPEC_SIZE+1];
	char fspec[MAX_FSPEC_SIZE+1];
	char fbase[MAX_FSPEC_SIZE+1];
	char defname[MAX_FSPEC_SIZE+1]="";
	char filter[132]="*.zip";
	int resp;
	int err;

	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);

	data.item_id=UGZ_PARTFILE;
	data.item_attr=UF_STYLER_VALUE;
	if((err=UF_STYLER_ask_value(dialog_id, &data)) != 0)	/* get part file name */
		UGError(err);

	if(*data.value.string)
	{
		if((err=uc4576(data.value.string, 2, dspec, fspec)) != 0)	/* split fname into directory and file-spec */
			UGError(err);
		if((err=uc4575(dspec, 0, "x.zip", filter)) != 0)	/* build filter from directory and '*.zip'		*/
			UGError(err);									/* this is a bad hack: uc4575 doesn't do *.zip	*/
		filter[strlen(filter)-5]='*';						/* so we add x.zip and replace the 'x' with '*'	*/
		if((err=uc4578(fspec, 2, fbase)) != 0)				/* remove .prt from file-spec */
			UGError(err);
		strcat(fbase, ".zip");
		if((err=uc4575(dspec, 0, fbase, defname)) != 0)		/* build default filename from dir, part and .zip	*/
			UGError(err);
	}
	err=UF_UI_create_filebox("Enter ZIP File Name", "New ZIP File", filter, defname, fname, &resp); 
	data.item_id=UGZ_ZIPFILE;
	data.item_attr=UF_STYLER_VALUE;
	data.value.string=fname;
	if((err=UF_STYLER_set_value(dialog_id, &data)) != 0)	/* place selected filename in zip input line */
		UGError(err);
	UpdateOkApply(dialog_id);
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	Callback-function for Ok-button
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_ok_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);
	if(! DoTheZIP(dialog_id))
		return(UF_UI_CB_CONTINUE_DIALOG); 
	UF_terminate();
    return(UF_UI_CB_EXIT_DIALOG);                           
}



/*******************************************************************
 *
 *	Callback-function for Apply-button
 *
 *	Input:	dialog_id		id of dialog-box
 *			client_data		not used
 *			callback_data	not used
 *
 *	Output:	Exit dialog
 *
 ******************************************************************/

int UGZ_apply_cb(int dialog_id, void *client_data, UF_STYLER_item_value_type_p_t callback_data)
{
	if(UF_initialize() != 0)
		return(UF_UI_CB_CONTINUE_DIALOG);
	DoTheZIP(dialog_id);
	UF_terminate();
	return(UF_UI_CB_CONTINUE_DIALOG); 
}



/*******************************************************************
 *
 *	UG-cleanup function
 *
 *	Input:	None
 *
 *	Output:	None
 *
 ******************************************************************/

extern void ufusr_cleanup(void)
{
    return;
}



/*******************************************************************
 *
 *	UG-unload function
 *
 *	Input:	None
 *
 *	Output:	Unload immediately
 *
 ******************************************************************/

int ufusr_ask_unload(void)
{
	return(UF_UNLOAD_IMMEDIATELY);
}



/*******************************************************************
 *
 *	Entry point, when set up via Menuscript
 *
 *	Input:	param		not used
 *			retcod		not used
 *			param_len	not used
 *
 *	Output:	None
 *
 ******************************************************************/

extern void ufsta (char *param, int *retcode, int rlen)
{
	int err;
 
    if((UF_initialize()) != 0) 
	{
		uc1601("UGZipC: No UG/Open API Execute - License available", TRUE);
		return;
	}
    if((err=UF_MB_add_styler_actions(actions)) != 0)
		UGError(err);
    UF_terminate();                            
    return;
}



/*******************************************************************
 *
 *	Entry point, when called via File/Execute UGOpen or Usertools
 *
 *	Input:	param		not used
 *			retcod		not used
 *			param_len	not used
 *
 *	Output:	None
 *
 ******************************************************************/

void ufusr(char *param, int *retcod, int param_len)
{
	int err;
    int resp;

	if((err=UF_initialize()) != 0) 
	{
		uc1601("UGZipC: No UG/Open API Execute - License available", TRUE);
		return;
	}
	if((err=UF_STYLER_create_dialog ("ugzipc.dlg", UGZ_cbs, UGZ_CB_COUNT, NULL, &resp)) != 0)
		UGError(err);
	UF_terminate();
}
